; This program exercises all of the 6811 instructions.
; The program was written to test the simulator.
;
; Each instruction is tested (in alphabetical order).
; Although this program "cries out" for "automatic
; generation" (i.e. a program that would generate
; this test program), I have chosen to write it by
; hand. (In particular, I have NOT even used
; "cut and paste", despite the obvious temptation,
; to write this code.)
;
;
; I have also ensured that this program works on real
; 68hc11 hardware.
;
; Although each instruction is tested, the tests are NOT
; exhaustive.  
;
; The tests are not independent; for example, the very
; first test (for the ABA instruction) uses other
; instructions.  A more sophisticated test suite could
; surely be built; but this dumb version gives me a
; platform I can use to add additional instruction tests
; when bugs in the basic engine are discovered.

; STUDENTS: This stand-alone program exercises EVERY*
; 6811 instruction. You can use it to discover what these
; instructions are and how each one is tested.  However,
; comments explaining what each instruction does are not
; included; nor do I explain why I test some side-effects
; of an instruction but not others.  Nonetheless, this
; program is included in the "samples" directory of
; "sim11" because it gives an example of every 6811
; instruction and because it demonstrates one technique
; for testing software.
; (* The WAI instruction is not tested.)

; Author: Ken Clowes
; Date: September 30, 2002

   org $6000

abaInsruction:
   ldaa #5
   ldab #6
   aba
   cmpa #11
   bne oopsAba
   bra abxInstruction

oopsAba:
   swi


abxInstruction:
   ldab #$ff
   ldx #$FF01
   abx

abyInstruction:
   aby

adcaInstruction:
   ldaa #$ff
   adda #1 ;should set carry
   bcc oopsAdca
   adca #0
   bra adcbInstruction

oopsAdca:
   swi


adcbInstruction:
    ldaa #6
    staa $00
    ldab #1
    adcb $00
    bcs oopsAdcb
    bra addaInstruction

oopsAdcb:
   swi

addaInstruction:
    ldaa #5
    ldy #0
    staa 1,y
    iny
    adda 0,y
    cmpa #10
    bne oopsAdda
    ldaa #$F0
    adda #$20
    bcc oopsAdda
    bvs oopsAdda
    bra addbInstruction

oopsAdda:
    swi

addbInstruction:
    clrb
    incb
    ldaa #$fe
    staa $7ffe
    addb $7ffe
    cmpb #$ff
    bne oopsAddb
    bra adddInstruction

oopsAddb:
    swi

adddInstruction:
    addd $6000
    bra andaInstruction

andaInstruction:
    anda #$ff
    bra andbInstruction

andbInstruction:
   andb #0
   tstb
   bne oopsAndb
   bra aslInstruction

oopsAndb:
   swi

aslInstruction:
   lda #2
   staa $2
   asl $2
   ldaa $2
   cmpa #4
   bne oppsAsl
   asla
   cmpa #8
   bne oppsAsl
   tab
   aslb
   cmpb #16
   bne oppsAsl
   bra asldInstruction

oppsAsl:
   swi

asldInstruction:
   ldd #$0102
   asld
   cmpd #$0204
   bne oopsAsld
   bra asrInstruction

oopsAsld:
   swi
asrInstruction:
   ldaa #2
   asra
   cmpa #1
   bne oopsAsr
   bra bccInstruction

oopsAsr:
   swi

bccInstruction:
   clra
   adda #$ff
   adda #2
   bcc oopsBcc
   bra BclrInstruction

oopsBcc
   swi

BclrInstruction:
   ldaa #$ff
   ldx #$7000
   staa $7000
   bclr  0,x $81
   ldaa $7000
   cmpa #$7E
   bne oopsBclr
   bra BcsInstruction

oopsBclr:
  swi

BcsInstruction:
   ldab #$ff
   addb #1
   bcs BeqInstruction

oopsBcs:
   swi

BeqInstruction:
;tested elsewhere

BgeInstruction:
   ldaa #9
   cmpa #10
   bge oopsBge
   bra BgtIinstruction

oopsBge:
   swi

BgtIinstruction:
   ldaa #2
   cmpa #2
   bgt oopsBgt
   bra BhiInstruction

oopsBgt:
   swi

BhiInstruction:
   ldaa #$80
   cmpa #$80
   bhi oopsBhi
   bra BhsInstruction

oopsBhi:
   swi

BhsInstruction:
   ldx #$F000
   cmpx #$F001
   bhs oopsBhs
   bra BitaInstruction

oopsBhs:
   swi

BitaInstruction:
   ldaa #%10101100
   staa $7000
   ldaa #%11001100
   bita $7000
   bvs oopsBita
   beq oopsBita
   bra BleInstruction

oopsBita:
   swi

BleInstruction:
   ldaa #$FF
   cmpa #$FE
   ble oopsBle
   bra BloInstruction

oopsBle:
  swi

BloInstruction:
   ldaa #$FF
   cmpa #$FF
   blo oopsBlo
   bra BlsInstruction

oopsBlo:
  swi

BlsInstruction:
   ldaa #1
   cmpa #0
   bls oopsBls
   bra BltInstruction

oopsBls:
   swi
   
BltInstruction:
   ldaa #$80
   cmpa #$81
   blt BmiInstruction

oopsBlt:
   swi

BmiInstruction:
   clra
   bmi oopsBmi
   inca
   bmi oopsBmi
   bra BneInstruction

oopsBmi:
   swi

BneInstruction:
    ldy #1234
    cmpy #1234
    bne oopsBne
    bra BplInstruction

oopsBne:
    swi

BplInstruction:
    clra
    deca
    bpl oopsBpl
    bra BraInstruction

oopsBpl:
    swi

BraInstruction:
    bra BrclrInstruction
oopsBra:
    swi

BrclrInstruction:
   ldx #$7000
   ldaa #2
   staa 0,x
   brclr 0,x 2 oopsBrclr
   bra BrnInstruction

oopsBrclr:
   swi

BrnInstruction:
   brn oopsBrn
   bra BrsetInstruction

oopsBrn:
   swi

BrsetInstruction:
   ldaa #3
   staa 3
   brset 3 %0001000 oopsBrset
   bra BsetInstruction

oopsBrset:
   swi

BsetInstruction:  
   ldaa #2
   staa 5
   bset  5 $81
   ldaa 5
   cmpa #$83
   bne oopsBset
   bra BsrInstruction

oopsBset:
  swi

BsrInstruction:
   bsr subBsr
   bra BvcInstruction
subBsr:
   rts

BvcInstruction:
   ldaa #$40
   adda #$40
   bvc oopsBvc
   bra BvsInstruction

oopsBvc:
   swi

BvsInstruction:
   ldaa #$30
   adda #$80
   bvs oopsBvs
   bra CbaInstruction

oopsBvs:
   swi

CbaInstruction:
   ldaa #1
   ldab #1
   cba
   bne oopsCba
   bra ClcInstruction

oopsCba:
   swi

ClcInstruction:
   ldab #$FF
   incb
   bcs oopsClc
   clc
   bcs oopsClc
   bra CliInstruction

oopsClc:
   swi
 
CliInstruction:
   cli
   tpa
   sei
   anda #%00010000
   bne oopsCli
   bra ClrInstruction

oopsCli:
   swi

ClrInstruction:
   clr $0
   clr $7000
   ldx #$7000
   clr 1,x
   tst 0
   bne oopsClr
   tst $7000
   bne oopsClr
   tst $7001
   bne oopsClr
   bra ClraInstruction

oopsClr:
   swi

ClraInstruction:
   clra
   cmpa #0
   bne oopsClra
   bra ClrbInstruction

oopsClra:
   swi

ClrbInstruction:
   clrb
   tstb
   bne oopsClrb
   bra ClvInstruction

oopsClrb:
  swi

ClvInstruction:
   ldaa #$ff
   tap
   clv
   bvs oopsClv
   bra CmpaInstruction

oopsClv:
   swi

  
CmpaInstruction:
    ldaa #$80
    cmpa #$79
    bls oopsCmpa
    bge oopsCmpa
    blo oopsCmpa
    bgt oopsCmpa
    beq oopsCmpa
    ldaa #1
    cmpa #2
    bhs oopsCmpa
    bhi oopsCmpa
    beq oopsCmpa
    ldaa #-1
    cmpa #-2
    beq oopsCmpa
    blt oopsCmpa
    blo oopsCmpa
    ldaa #1
    cmpa #-1
    beq oopsCmpa
    blt oopsCmpa
    ble oopsCmpa
    bhi oopsCmpa
    bhs oopsCmpa
    bra CmpbInstruction

oopsCmpa:
    swi

CmpbInstruction:
    ldab #10
    cmpb #9
    beq oopsCmpb
    blt oopsCmpb
    blo oopsCmpb
    bra ComInstruction

oopsCmpb:
    swi

ComInstruction:
   clr 0
   com 0
   tst 0
   bpl oopsCom
   beq oopsCom
   bra ComaInstruction

oopsCom:
   swi

ComaInstruction:
   clra
   inca
   coma
   bpl oopsComa
   beq oopsComa
   cmpa #$FE
   bne oopsComa
   bra CombInstruction

oopsComa:
   swi

CombInstruction:
   clrb
   comb
   cmpb #-1
   bne oopsComb
   bra CpdInstruction

oopsComb:
   swi

CpdInstruction:
   ldd #$F000
   cmpd #$EFFF
   blo oopsCpd
   beq oopsCpd
   ble oopsCpd
   bra CpxInstruction

oopsCpd:
   swi

CpxInstruction:
   ldx #500
   cpx #600
   bgt oopsCpx
   bra CpyInstruction

oopsCpx:
   swi

CpyInstruction:
   ldy #0
   cmpy #0
   bne oopsCpy
   bra DaaInstruction

oopsCpy:
   swi


DaaInstruction:
   ldaa #$49
   adda #1
   daa
   cmpa #$50
   bne oopsDaa
   bmi oopsDaa
   adda #$35
   daa
   bpl oopsDaa
   adda #$15
   daa
   bne oopsDaa
   bcc oopsDaa
   bra DecInstruction
   

oopsDaa:
   swi

DecInstruction:
   clr 0
   dec 0
   bpl oopsDec
   beq oopsDec
   bge oopsDec
   ldaa 0
   cmpa #-1
   bne oopsDec
   bra DecaInstruction
oopsDec:
   swi

DecaInstruction:
   ldaa #$81
   deca
   bpl oopsDeca
   deca
   bmi oopsDeca
   cmpa #$7F
   bne oopsDeca
   bra DecbInstruction

oopsDeca
   swi

DecbInstruction:
   ldab #2
   decb
   cmpb #1
   bne oopsDecb
   bra DesInstruction

oopsDecb:
   swi

DesInstruction:
   tsx
   stx 0
   des
   tsx
   xgdx
   subd 0
   cmpd #-1
   bne oopsDes
   bra DexInstruction

oopsDes:   
   swi

DexInstruction:
  ldx #1234
  dex
  cpx #1233
  bne oopsDex
  bra DeyInstruction

oopsDex:
  swi

DeyInstruction:
   ldy #1
   dey
   bne oopsDey
   bra EoraInstruction

oopsDey:
   swi

EoraInstruction:
   ldaa #$F0
   eora #$A3
   bmi oopsEora
   cmpa #$53
   bne oopsEora
   bra EorbInstruction

oopsEora:
   swi

EorbInstruction:
   ldab #$72
   eorb #$14
   bmi oopsEorb
   cmpb #$66
   bne oopsEorb
   bra FdivInstruction

oopsEorb:
   swi

FdivInstruction:
    ldd #1500
    ldx #6000
    fdiv
    cmpx #$4000
    bne oopsFdiv
    cpd #0
    bne oopsFdiv
    bra IdivInstruction

oopsFdiv:
   swi

IdivInstruction:
   ldd #23
   ldx #10
   idiv
   bra IncInstruction

IncInstruction:
   clr 0
   inc 0
   inc 0
   clr 1
   ldx 0
   cmpx #$0200
   bne oopsInc
   bra IncaInstruction

oopsInc:
   swi


IncaInstruction:
   clra
   coma
   inca
   bne oopsInca
   inca
   cmpa #1
   bne oopsInca
   bra IncbInstruction

oopsInca:
   swi

IncbInstruction:
   incb
   bra InsInstruction

InsInstruction:
   tsx
   stx $7000
   ins
   tsx
   xgdx
   subd $7000
   cpd #1
   bne oopsIns
   bra InxInstruction

oopsIns:
   swi

InxInstruction:
   ldx #$FFFF
   inx
   inx
   cpx #1
   bne oopsInx
   bra InyInstruction

oopsInx:
   swi

InyInstruction:
   ldy #0
   iny
   cpy #1
   bne oopsIny
   bra JmpInstruction

oopsIny:
   swi

JmpInstruction:
   jmp JsrInstruction

oopsJmp:
   swi
JsrInstruction:
   lds #$0041
   ldaa #$39 ; rts instruction
   staa $7000
   jsr $7000
   bra LdaaInstruction

LdaaInstruction:
   sev
   ldaa #0
   bne oopsLdaa
   bmi oopsLdaa
   bvs oopsLdaa
   bra LdabInstruction

oopsLdaa
   swi

LdabInstruction:
   sev
   ldab #-5
   beq oopsLdab
   bpl oopsLdab
   bvs oopsLdab
   bra LddInstruction

oopsLdab:
   swi

LddInstruction:
  sev
  ldd #$8800
  beq oopsLdd
  bpl oopsLdd
  bvs oopsLdd
  bra LdsInstruction

oopsLdd:
  swi

LdsInstruction:
   sev
   lds #$ffff
   bvs oopsLds
   tsx
   cpx #0
   bne oopsLds
   lds #$0041
   bra LdxInstruction

oopsLds:
   swi

LdxInstruction:
   sev
   ldx #0
   bne oopsLdx
   bmi oopsLdx
   bvs oopsLdx
   bra LdyInstruction

oopsLdx:
   swi

LdyInstruction:
   sev
   ldy #$ffff
   bvs oopsLdy
   bra LslInstruction

oopsLdy:
   swi

LslInstruction:
   clc
   ldaa #%11000000
   staa 0
   sev
   lsl 0
   bcc oopsLsl
   bpl oopsLsl
   bvs oopsLsl
   bra LslaInstruction

oopsLsl:
   swi

LslaInstruction:
  clc
  ldaa #%11000101
  sev
  lsla
  bcc oopsLsla
  bpl oopsLsla
  bvs oopsLsla
  bra LslbInstruction

oopsLsla:
  swi

LslbInstruction:
  ldab #%00110011
  clc
  sev
  lslb
  bcs oopsLslb
  bmi oopsLslb
  bvs oopsLslb
  bra LsldInstruction

oopsLslb:
  swi

LsldInstruction:
  ldd #$4210
  lsld
  cmpd #$8420
  bne oopsLsld
  bra LsrInstruction

oopsLsld:
  swi
LsrInstruction:
   clr 0
   inc 0
   inc 0
   lsr 0
   ldaa 0
   cmpa #1
   bne oopsLsr
   bra LsraInstruction

oopsLsr:
   swi
LsraInstruction:
   ldaa #8
   lsra
   cmpa #4
   bne oopsLsra
   lsra
   lsra
   ldab #5
   tstb
   lsra
   bne oopsLsra
   bra LsrbInstruction

oopsLsra:
   swi

LsrbInstruction:
   ldab #2
   lsrb
   cmpb #1
   bne oopsLsrb
   bra LsrdInstruction

oopsLsrb:
   swi

LsrdInstruction:
   ldd #$0100
   lsrd
   cmpd #$0080
   bne oopsLsrd
   bra MulInstruction

oopsLsrd:
   swi

MulInstruction:
   ldaa #25
   ldab #12
   mul
   cmpd #300
   bne oopsMul
   bra NegInstruction

oopsMul:
   swi

NegInstruction:
   clr 0
   inc 0
   neg 0
   ldaa 0
   cmpa #-1
   bne oopsNeg
   bra NegaInstruction

oopsNeg:
   swi

NegaInstruction:
   ldaa #$7f
   nega
   cmpa #$81
   bne oopsNega
   bra NegbInstruction

oopsNega:
   swi

NegbInstruction:
  ldab #1
  negb
  cmpb #-1
  bne oopsNegb
  bra NopInstruction

oopsNegb:
  swi
 
NopInstruction:
   nop

OraaInstruction:
   ldaa #$F2
   oraa #$84
   cmpa #$F6
   bne oopsOraa
   bra OrabInstruction

oopsOraa:
   swi

OrabInstruction:
  ldab #$55
  orab #$AA
  cmpb #-1
  bne oopsOrab
  bra PshaInstruction

oopsOrab:
  swi

PshaInstruction:
PulaInstruction:
   ldaa #5
   psha
   clra
   pula
   cmpa #5
   bne oopsPsha
   bra PshbInstruction

oopsPsha:
   swi

PshbInstruction:
PulbInstruction:
   ldab #7
   pshb
   incb
   pulb
   cmpb #7
   bne oopsPshb
   bra PshxInstruction

oopsPshb:
  swi


PshxInstruction:
PshyInstruction:
PulxInstruction:
PulyInstruction:
   ldx #1234
   ldy #5678
   pshx
   pshy
   dex
   iny
   puly
   pulx
   cmpx #1234
   bne oopsPshx
   cmpy #5678
   bne oopsPshy   
   bra RolInstruction

oopsPshx:
oopsPshy:
   swi

RolInstruction:
   ldaa #1
   staa 0
   sec
   rol 0
   ldaa 0
   cmpa #3
   bne oopsRol
   bra RolaInstruction

oopsRol:
   swi

RolaInstruction:
  ldaa #2
  sec
  rola
  cmpa #5
  bne oopsRola
  bra RolbInstruction

oopsRola:
  swi

RolbInstruction:
  ldab #$81
  clc
  rolb
  bcc oopsRolb
  cmpb #2
  bne oopsRolb
  bra RorInstruction

oopsRolb:
  swi
 
RorInstruction:
  ldaa #$81
  staa 0
  sec
  ror 0
  bcc oopsRor
  ldaa 0
  cmpa #$C0
  bne oopsRor
  bra RoraInstruction

oopsRor:
  swi

RoraInstruction:
   clra
   sec
   rora
   bpl oopsRora
   bra RorbInstruction

oopsRora:
   swi
RorbInstruction:
   ldab #4
   sec
   rorb
   cmpb #$82
   bne oopsRorb
   bra RtiInstruction

oopsRorb:
   swi

RtiInstruction:
   jsr dummyISR

   bra RtsInstruction

dummyISR:
   pshy
   pshx
   pshb
   psha
   tpa
   psha
   rti

;See JsrInstruction
RtsInstruction:

SbaInstruction:
  ldaa #$ff
  ldab #$ff
  sba
  bvs oopsSba
  bcs oopsSba
  bra SbcaInstruction

oopsSba:
  swi

SbcaInstruction:
  ldaa #200
  sec
  sbca #100
  cmpa #99
  bne oopsSbca
  bra SbcbInstruction

oopsSbca:
  swi

SbcbInstruction:
   sec
   ldab #6
   sbcb #5
   bne oopsSbcb
   bra SecInstruction

oopsSbcb:
   swi


SecInstruction:
   sec
   bcc oopsSec
   bra SeiInstruction

oopsSec:
   swi

SeiInstruction:
   sei
   tpa
   anda #%00010000
   beq oopsSei
   bra SevInstruction

oopsSei:
   swi

SevInstruction:
   sev
   bvc oopsSev
   bra StaaInstruction

oopsSev:
   swi

StaaInstruction:
  clra
  staa $7001
  tst $7001
  bne oopsStaa
  bra StabInstruction

oopsStaa:
  swi

StabInstruction:
  clrb
  incb
  stab $7001
  dec $7001
  tst $7001
  bne oopsStab
  bra StdInstruction

oopsStab:
  swi

StdInstruction:
  ldd #$2468
  std $7000
  ldd #0
   ldd $7000
   cpd #$2468
   bne oopsStd
   bra StopInstruction

oopsStd:
   swi

StopInstruction:
;already tested

StsInstruction:
   lds #$0030
   sts $7000
   pshx
   lds $7000
   tsx
   cpx #$31
   bne oopsSts
   lds #$0041
   bra StxInstruction

oopsSts:
   swi

StxInstruction:
   ldx #321
   stx $7000
   dex
   ldx $7000
   cpx #321
   bne oopsStx
   bra StyInstruction

oopsStx:
   swi

StyInstruction:
  ldy #0
  sty $7000
  tst $7000
  bne oopsSty
  tst $7001
  bne oopsSty
  bra SubaInstruction

oopsSty:
  swi

SubaInstruction:
  ldaa #5
  suba #6
  bpl oopsSuba
  beq oopsSuba
  bvs oopsSuba
  bcc oopsSuba
  bra SubbInstruction

oopsSuba:
  swi

  
SubbInstruction:
  ldab #127
  subb #-5
  bcc oopsSubb
  bvc oopsSubb
  bra SubdInstruction

oopsSubb:
  swi

SubdInstruction:
   ldd #$7fff
   subd #$1000
   bcs oopsSubd
   bvs oopsSubd
   bra SwiInstruction

oopsSubd:
  swi

SwiInstruction:
   ;already tested

TabInstruction:
   ldd #$1122
   tab
   cmpb #$11
   bne oopsTab
   bra TapInstruction

oopsTab:
   swi

TapInstruction:
    ldaa #$7f
    tap
    bpl oopsTap
    bne oopsTap
    bcc oopsTap
    bvc oopsTap
    bra TbaInstruction

oopsTap:
   swi

TbaInstruction:
  ldd #$5566
  tba
  cmpa #$66
  bne oopsTba
  bra TpaInstruction

oopsTba:
  swi

;TestInstruction:

TpaInstruction:
   ldaa #0
   adda #$FF
   sei
   clc
   clv
   tpa
   anda #$3f
   cmpa #$18
   bne oopsTpa
   bra TstInstruction

oopsTpa:
   swi

TstInstruction:
   clr 0
   tst 0
   bne oopsTst
   bra TstaInstruction

oopsTst:
   swi

TstaInstruction:
   ldaa $-1
   tsta
   bpl oppsTsta
   bra TstbInstruction

oppsTsta:
   swi

TstbInstruction:
   ldab #100
   addb #100
   bvc oopsTstb
   clra
   tstb
   bpl oopsTstb
   beq oopsTstb
   bra TsxInstruction

oopsTstb:
   swi

TsxInstruction:
TxsInstruction:
TsyInstruction:
TysInstruction:
   lds #$0020
   ldx #0
   tsx
   lds #0
   txs
   tsy
   cpy #$0021
   bne oopsTsx
   bra WaiInstruction

oopsTsx:
   swi

WaiInstruction:
;Wai not yet implemented

XgdxInstruction:
  ldd #1234
  ldx #5678
  xgdx
  cpd #5678
  bne oopsXgdx
  bra XgdyInstruction

oopsXgdx:
  swi

XgdyInstruction:
  ldd #0
  ldy #-1
  xgdy
  cmpd #-1
  bne oopsXgdy
  cpy #0
  bne oopsXgdy
  bra AllDOne

oopsXgdy:
  swi

AllDOne:

   stop
   nop
   nop


   swi


   end

